% TP4 S5 2023-2024 Optimisation non linaire avec contraintes

disp('test fmincon');
% tests de fmincon sur les problmes dont les donnees sont dans fobj et
% nonlincon (mthode des points intrieurs par dfaut)

% QUELQUES INDICATIONS POUR LE CHOIX DES OPTIONS (cf doc fmincon)
% Pour connatre toutes les options par dfaut :
% options = optimoptions(@fmincon)
% La mthode par dfaut est la mthode des points intrieurs

% Quelques suggestions de modifications :
% pour un affichage dtaill des itrs :
% options = optimoptions(@fmincon,options,'Display' ,'iter_detailed');
% remarque : le paramtre options juste aprs le paramtre @fmincon permet
% de conserver les options dj choisies et d'y ajouter une nouvelle par
% exemple ici la modification des affichages de rsultats avec l'option
% 'Displays'. Voir l'exemple qui suit avec l'option 'Maxiter'.

% pour modifier le nombre maximum d'itrations :
% en conservant les options prcdemment choisies :
% options = optimoptions(@fmincon,options,'Maxiter' ,'2000');
% ou en repartant des options par dfaut :
% options = optimoptions(@fmincon,'Maxiter' ,'2000');

% choix des options
options = optimoptions(@fmincon);% pour partir des options par dfaut
options = optimoptions(@fmincon,options,'TolX' ,1e-4);
% options pour utiliser les gradients de la fonction objective et des
% contraintes
options = optimoptions(@fmincon,options,'SpecifyObjectiveGradient',true);
options = optimoptions(@fmincon,options,'SpecifyConstraintGradient',true);

choixpb = input('Probleme n : ');

% cadre des graphes [a,b]x[c,d] suivant le pb
a = [-5,-10,-15]; b = [5,10,15];
c = [-5,-10,-15]; d = [5,10,20];
% trac des courbes de niveau 
clf; hold on;
mx = 30; my = 30;
xx = linspace(a(choixpb),b(choixpb),mx); yy = linspace(c(choixpb),d(choixpb),my);
zzsurf = zeros(my,mx);
for i=1:mx
     for j=1:my
         zzsurf(j,i)=fobj([xx(i),yy(j)],choixpb);   
     end
end
[px,py] = gradient(zzsurf,.5,.5);
contour(xx,yy,zzsurf,32);
quiver(xx,yy,px,py);
colormap(hsv);
title(['Problme ',int2str(choixpb)]);

lb = [];
ub = [];

% initialisation suivant le problme
switch choixpb
    case 1
        % trac du contour de K
        t = linspace(0,pi/2,50); plot(cos(t),sin(t),'r');
        plot([0,0,1],[1,0,0],'r');
        % trac du point de min
        ptmin = plot(1,0,'r*');
        % choix du point initial
        x0 = input('Point initial x0 (choisir pour tester [5 5] ou [0 0.5])= '); 
        ptinit = plot(x0(1),x0(2),'b*');
        lb = [0 0];
    case 2
        % trac du cercle de centre (0,0) de rayon 1
        t = linspace(0,2*pi,50);
        plot(cos(t),sin(t),'r');
        % trac des points de min
        ptmin = plot(0.9239,0.3827,'r*');plot(-0.9239,-0.3827,'r*');
        % choix du point initial
        x0 = input('Point initial x0 (choisir pour tester [10 10] ou [0 -2]) = ');
        ptinit = plot(x0(1),x0(2),'b*');
        
    case 3
        % trac du cercle de centre (-5,10) de rayon 10
        t = linspace(0,2*pi,50);
        plot(-5+10*cos(t),10+10*sin(t),'r');
        % trac du cercle de centre (5,0) de rayon 10
        plot(5+10*cos(t),10*sin(t),'r');
        % trac des points de min
        ptmin = plot(8.3307,9.4112,'r*');
        ptminlocal = plot(-5,0,'rsquare'); plot(1.6193,-9.4112,'rsquare');
        plot(-14.5140,13.0794,'rsquare');
        % choix du point initial
        x0 = input('Point initial x0 (choisir pour tester [-15 -15], [-5 -15], [0 -15], [-15 15], [15 15]) = ');
        ptinit = plot(x0(1),x0(2),'b*');
end

% rsolution par fmincon
disp('Rsultats par fmincon :');
fun = @(x) fobj(x, choixpb);
nonlcon = @(x) nonlincon(x, choixpb);
[x,fval,exitflag,output,lambda] = fmincon(fun,x0,[],[],[],[],lb,ub,nonlcon,options);

if(exitflag==1)
    disp('Itrations : '); disp(output.iterations);
    disp('x = '); disp(x);
    disp('fval = '); disp(fval);
end

% trac du point obtenu
ptfmincon = plot(x);

% rsolution par gradient projet pour les problmes 1 et 2
% et lgendes de graphes obtenus
if (choixpb < 3)  
    disp('Rsultats par la mthode du gradient projet :');
    % A COMPLETER
 
    % trac du point obtenu
    % ptgradientprojete = plot(...);
    legend([ptmin,ptinit,ptfmincon,ptgradientprojete],'solution','point initial','solution pt intrieur','solution gradient projete');
else
    legend([ptmin,ptminlocal,ptinit,ptfmincon],'min global','min local','point initial','solution pt intrieur');
end
